#!/usr/bin/env python
# -*- coding: utf-8 -*-

import numpy as np
from dataclasses import dataclass, field
from scipy.signal import peak_widths


@dataclass
class Peak:
    """
    A dataclass to represent and manage the properties of a detected peak.

    Attributes
    ----------
    time : float
        The time at which the peak occurs.
    height : float
        The height (amplitude) of the peak.
    width : float
        The width of the peak at half maximum.
    area : float
        The area under the peak, calculated post-initialization.
    """
    times: np.ndarray
    signal: np.ndarray
    index: float
    height: float
    width: float
    area: float = field(init=False, default=None)

    def __post_init__(self):
        self.time = self.times[self.index]
        # self.calculate_widths()

    def calculate_widths(self) -> None:
        """
        Calculates the widths of the detected peaks at half maximum height.
        Widths are calculated only after peaks are detected using the `find_peaks` method.
        """
        dt = self.times[1] - self.times[0]  # Time step

        self.widths = peak_widths(
            self.signal.magnitude,
            [self.index],
            rel_height=0.5
        )

        print(self.widths)

        start_idx = int((self.time - self.width/2) / dt)
        end_idx = int((self.time + self.width/2) / dt)
        self.area = np.trapz(self.signal[start_idx:end_idx], self.time[start_idx:end_idx])

